package ch.lanter.vajanoid;

import java.awt.image.BufferedImage;
import java.net.URL;

import javax.imageio.ImageIO;

/**
 * This class contains the needed images and provides functionality to load
 * those. This is a Singleton.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class ImageCenter {
	
	private static ImageCenter instance;

	public static ImageCenter getInstance() {
		return instance;
	}
	
	static {
		instance = new ImageCenter();
	}

	public BufferedImage imgBall; // normal (iron) ball
	public BufferedImage imgBall2; // invincible (golden) ball
	public BufferedImage imgShot;
	public BufferedImage imgPadLeft;
	public BufferedImage imgPadMiddle;
	public BufferedImage imgPadRight;
	
	private ImageCenter() {
		imgBall = loadImage("/ress/ball.png");
		imgBall2 = loadImage("/ress/ball2.png");
		imgShot = loadImage("/ress/shot.png");
		loadPadImages();
	}
	
	private void loadPadImages() {
		BufferedImage image = loadImage("/ress/pad.png");
		imgPadLeft =	new BufferedImage(4,9,BufferedImage.TYPE_INT_ARGB);
		imgPadMiddle =	new BufferedImage(1,9,BufferedImage.TYPE_INT_ARGB);
		imgPadRight =	new BufferedImage(3,9,BufferedImage.TYPE_INT_ARGB);
		imgPadLeft	.getGraphics().drawImage(image,0,0,4,9,0,0,4,8,null);
		imgPadMiddle.getGraphics().drawImage(image,0,0,1,9,4,0,5,8,null);
		imgPadRight	.getGraphics().drawImage(image,0,0,3,9,5,0,8,8,null);
	}
	
	public BufferedImage loadImage(String path) {
		BufferedImage image = null;
		try {
			URL url = ImageCenter.class.getResource(path);
			image = ImageIO.read(url);
		} catch (Exception e) {
			System.err.println("Error while reading the Image file "+path);
			e.printStackTrace();
		}
		return image;
	}
	
}
