package ch.lanter.vajanoid;

import java.awt.Graphics;
import java.awt.Rectangle;

/**
 * @author Martin Lanter, LanterSoft 2010
 */
public class Item implements Positionable {

	private static final long serialVersionUID = 201001130142L;

	private float x,y; // position
	private int width, height;

	private ItemType type;

	private float speed; // speed in pixels per second
	private boolean catched; // true if this item has been catched with the pad

	public Item(int x, int y, ItemType type) {
		this.width = 16;
		this.height = 8;
		this.x = x - width/2;
		this.y = y - height/2;
		this.type = type;
		this.speed = 200f;
		this.catched = false;
	}

	/**
	 * update the position according to the passed time dt
	 */
	public void step(long dt) {
		y = y + speed/1000*dt;
	}

	public ItemType getType() {
		return type;
	}

	/**
	 * @return - true it pos and this item intersect
	 */
	public boolean intersects(Positionable pos) {
		return (!catched && getBounds().intersects(pos.getBounds()));
	}

	public void setCatched() {
		catched = true;
	}

	public boolean isCatched() {
		return catched;
	}

	@Override
	public void paint(Graphics g) {
		if (catched) return;
		int xx = getX();
		int yy = getY();
		g.drawImage(type.getLeft(),xx,yy,null);
		g.drawImage(type.getMiddle(),xx+4,yy,width-8,8,null);
		g.drawImage(type.getRight(),xx+width-4,yy,null);
	}

	public Rectangle getBounds() {
		return new Rectangle(getX(),getY(),width,height);
	}

	public int getX() {
		return Math.round(x);
	}

	public int getY() {
		return Math.round(y);
	}

	public int getWidth() {
		return Math.round(width);
	}

	public int getHeight() {
		return Math.round(height);
	}
}
