package ch.lanter.vajanoid;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URL;
import java.util.Random;

import javax.imageio.ImageIO;

import ch.lanter.vajanoid.block.BlockImage;

/**
 * Enumeration of types an item is of.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public enum ItemType {
	
	// Instances of enum BlockImage
	YELLOW	(0),
	ORANGE	(1),
	RED		(2),
	PURPLE	(3),
	BLUE	(4),
	GREEN	(5),
	BLACK	(6),
	WHITE	(7);
	
	private static String imageDir = "/ress/items/"; // default path to the directory
	private static BufferedImage[] imagesL = new BufferedImage[8];
	private static BufferedImage[] imagesM = new BufferedImage[8];
	private static BufferedImage[] imagesR = new BufferedImage[8];

	private static int[] props; // probability for an item to be released
	
	private static boolean isInit = false;
	
	static {
		init();
	}
	
	/**
	 * Load all images and setup the probabilities for every type
	 */
	public static void init() {
		if (isInit) return;
		else isInit = true;
		loadImagesFromDir(imageDir); // load the block images from the default directory
		props = new int[8];
		props[0] = 10; // yellow
		props[1] = props[0]+ 2; // orange
		props[2] = props[1]+ 4; // red
		props[3] = props[2]+20; // purple
		props[4] = props[3]+ 5; // blue
		props[5] = props[4]+10; // green
		props[6] = props[5]+ 5; // black
		props[7] = props[6]+ 2; // white
	}
	
	/**
	 * @return - a random itemtype
	 */
	public static ItemType getRandomItemType() {
		Random rand = new Random();
		int r = rand.nextInt(props[props.length-1]);
		if (r<props[0]) return YELLOW;
		else if (r<props[1]) return ORANGE;
		else if (r<props[2]) return RED;
		else if (r<props[3]) return PURPLE;
		else if (r<props[4]) return BLUE;
		else if (r<props[5]) return GREEN;
		else if (r<props[6]) return BLACK;
		else if (r<props[7]) return WHITE;
		throw new RuntimeException("impossible r computed: "+r+", while props[length-1]="+props[props.length-1]);
	}
	
	/**
	 * Load the block Images from the given directory
	 * @param dir -  the directory
	 */
	private static void loadImagesFromDir(String dir) {		
		loadImage(dir+"item_yellow.png",0);
		loadImage(dir+"item_orange.png",1);
		loadImage(dir+"item_red.png",2);
		loadImage(dir+"item_purple.png",3);
		loadImage(dir+"item_blue.png",4);
		loadImage(dir+"item_green.png",5);
		loadImage(dir+"item_black.png",6);
		loadImage(dir+"item_white.png",7);
	}
	
	/**
	 * Load an Image from the given path
	 * @param path - the path
	 * @return - the Image
	 */
	private static void loadImage(String path, int pointer) {
		try {
			URL url = BlockImage.class.getResource(path);
			BufferedImage image = ImageIO.read(url);
			imagesL[pointer] = new BufferedImage(4,8,BufferedImage.TYPE_INT_ARGB);
			imagesM[pointer] = new BufferedImage(1,8,BufferedImage.TYPE_INT_ARGB);
			imagesR[pointer] = new BufferedImage(3,8,BufferedImage.TYPE_INT_ARGB);
			imagesL[pointer].getGraphics().drawImage(image, 0,0,4,8, 0,0,4,8,null);
			imagesM[pointer].getGraphics().drawImage(image, 0,0,1,8, 4,0,5,8,null);
			imagesR[pointer].getGraphics().drawImage(image, 0,0,3,8, 5,0,8,8,null);
		} catch (Exception e) {
			System.err.println("Fehler beim Lesen der Bild-Datei "+new File(path).getAbsolutePath());
			e.printStackTrace();
		}
	}
	
	private int pointer; // pointer to the image of a type
	
	/**
	 * Constructor
	 */
	private ItemType(int pointer) {
		this.pointer = pointer;
	}
	
	public Image getLeft() {
		return imagesL[pointer];
	}
	
	public Image getMiddle() {
		return imagesM[pointer];
	}
	
	public Image getRight() {
		return imagesR[pointer];
	}
}
