package ch.lanter.vajanoid;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import ch.lanter.vajanoid.BallEvent.TYPE;

/**
 * @author Martin Lanter, LanterSoft 2010
 */
public class Pad implements Positionable {

	private static final long serialVersionUID = 201001130143L;

	public static final int DEFAULT_WIDTH = 48;

	private int width;
	private int height;
	private int max;
	private transient Image imgLeft;
	private transient Image imgMiddle;
	private transient Image imgRigth;

	private int y;
	private int x;

	private int shots; // how many shots per click

	private double angleMin, angleMax; // min and max angle a ball must have when padding it

	private List<Ball> boundedBalls;
	private List<Integer> boundx; // x coordinates whre the bounded balls are on the pad

	public Pad(int max) {
		this.max = max;
		this.width = DEFAULT_WIDTH;
		this.height = 12;
		this.angleMin = Math.PI/180*10; // 10
		this.angleMax = Math.PI/180*70; // 70
		this.imgLeft = ImageCenter.getInstance().imgPadLeft;
		this.imgMiddle = ImageCenter.getInstance().imgPadMiddle;
		this.imgRigth = ImageCenter.getInstance().imgPadRight;
		this.boundedBalls = new ArrayList<Ball>();
		this.boundx = new ArrayList<Integer>();
		this.shots = 0;
	}

	/**
	 * Copy Constructor
	 */
	public Pad(Pad pad) {
		this.x = pad.x;
		this.y = pad.y;
		this.max = pad.max;
		this.height = pad.height;
		this.imgLeft = pad.imgLeft;
		this.imgMiddle = pad.imgMiddle;
		this.imgRigth = pad.imgRigth;
		this.width = DEFAULT_WIDTH;
		this.angleMin = Math.PI/180*10; // 10
		this.angleMax = Math.PI/180*70; // 70
		this.boundedBalls = new ArrayList<Ball>();
		this.boundx = new ArrayList<Integer>();
		this.shots = 0;
		moveTo(this.x+width/2);
	}

	/**
	 * Pad the ball. Called if the ball and the pad intersect
	 */
	public void padBall(Ball ball) {
		if (ball.bound) return; // if the ball is bound, there is nothing to pad
		boolean contact = getBounds().intersects(ball.getBounds());
		if (contact) {
			if (!ball.bind) {
				float diff = Math.abs( getPosX()+getPadWidth()/2 - ball.x - ball.width/2);
				float factor = diff / (getPadWidth()/2 + ball.width/2);
				double angle = angleMin+(angleMax-angleMin)*factor;
				double velocity =  ball.velocity;// Math.sqrt(ball.vx*ball.vx + ball.vy*ball.vy);
				if (ball.y+ball.height+3>y) ball.y = y-ball.height+3;
				if (getPosX()+getPadWidth()/2<ball.x+ball.width/2) { // ball to the right of the pad
					ball.vy = (float) -(velocity*Math.sin(Math.PI/2-angle));
					ball.vx = (float) (velocity*Math.cos(Math.PI/2-angle));
				} else {
					ball.vy = (float) -(velocity*Math.sin(Math.PI/2+angle));
					ball.vx = (float) (velocity*Math.cos(Math.PI/2+angle));
				}
				int xx = this.x+width/2;
				int yy = this.y+height;
				ball.launchEvent(new BallEvent(xx,yy,TYPE.BALL_PADDED));
			} else {
				bindBall(ball);
			}
		}
	}

	/**
	 * Bind the given ball to the pad
	 */
	public void bindBall(Ball ball) {
		if (!boundedBalls.contains(ball)) { // new ball to bind
			boundedBalls.add(ball);
			boundx.add((int) ball.x - getPosX());
			ball.y = getPosY()-ball.height+3;
			ball.bound = true;
		}
	}

	public void adjust(Point locView) {
		// Not allowed in applets:
//		Point locMouse = MouseInfo.getPointerInfo().getLocation();
//		if (locMouse.x<locView.x+this.getPadWidth()/2) {
//			this.setPosX(0);
//		} else if (locMouse.x>locView.x+max-this.getPadWidth()/2) {
//			this.setPosX(max-this.getPadWidth());
//		} else {
//			this.setPosX(locMouse.x-locView.x-this.getPadWidth()/2);
//		}
	}

	public void expandWidth() {
		if (width<DEFAULT_WIDTH) {
			x = x-(DEFAULT_WIDTH-width)/2;
			width = DEFAULT_WIDTH;
		} else {
			width = width + 20;
			x = x - 10;
		}
	}

	public void reduceWidth() {
		if (width>DEFAULT_WIDTH) {
			x = x-(DEFAULT_WIDTH-width)/2;
			width = DEFAULT_WIDTH;
		} else {
			x = x + width/8;
			width = width - width/4;
		}
	}

	public void releaseBoundedBalls() {
		for (int i=0;i<boundedBalls.size();i++) {
			Ball ball = boundedBalls.get(i);
			ball.bound = false;
			boolean bindAgain = ball.bind;
			ball.bind = false;
			padBall(ball);
			ball.bind = bindAgain;
		}
		boundedBalls.clear();
		boundx.clear();
	}

	/**
	 * Move the pad to the given x coordinate if allowed
	 */
	public void moveTo(int x) {
		if (x < this.getPadWidth()/2) {
			this.setPosX(0);
		} else if (x > max-this.getPadWidth()/2) {
			this.setPosX(max-this.getPadWidth());
		} else {
			this.setPosX(x-this.getPadWidth()/2);
		}
	}

	/**
	 * Set the pad to the given x coordinate
	 */
	public void setPosX(int x) {
		this.x = x;
		for (int i=0;i<boundedBalls.size();i++) {
			Ball ball = boundedBalls.get(i);
			ball.x = getPosX()+boundx.get(i);
		}
	}

	public void setPosY(int y) {
		this.y = y;
	}

	public int getPosX() {
		return x;
	}

	public int getPosY() {
		return y;
	}

	public int getPadWidth() {
		return width;
	}

	public void setShots(int shots) {
		this.shots = shots;
	}

	@Override
	public void paint(Graphics g) {
		drawCanons(g);
		int xx = getX();
		int yy = getY();
		g.drawImage(imgLeft,xx+0,yy+3,null);
		g.drawImage(imgMiddle,xx+4,yy+3,width-7,imgMiddle.getHeight(null),null);
		g.drawImage(imgRigth,xx+getWidth()-3,yy+3,null);
	}

	private void drawCanons(Graphics g) {
		for (int i=0;i<shots;i++) {
			int xx = getX()+(i+1)*width/(shots+1);
			int yy = getY();
			g.setColor(new Color(0,200,0));
			g.fillPolygon(new int[] {xx-4,xx,xx+4}, new int[] {yy+4,yy+0,yy+4}, 3);
			g.setColor(new Color(0,100,0));
			g.drawPolygon(new int[] {xx-4,xx,xx+4}, new int[] {yy+4,yy+0,yy+4}, 3);
		}
	}

	public Rectangle getBounds() {
		return new Rectangle(getX(),getY(),width,height);
	}

	public int getX() {
		return Math.round(x);
	}

	public int getY() {
		return Math.round(y);
	}

	public int getWidth() {
		return Math.round(width);
	}

	public int getHeight() {
		return Math.round(height);
	}
}
