package ch.lanter.vajanoid;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;

/**
 * @author Martin Lanter, LanterSoft 2010
 */
public class Shot implements Positionable, Eventable {

	private static final long serialVersionUID = 201001130144L;

	private Model model;

	protected float x,y;
	protected int width, height;

	private boolean catched; // true if the item has been catched

	private int speed = 200; // speed in pixels per second

	public Shot(Model model, int x, int y) {
		this.model = model;
		this.width = 4;
		this.height = 4;
		this.x = x - width/2;
		this.y = y - height/2;
		this.catched = false;
	}

	public void step(long dt) {
		y = y - speed*dt/1000f;
	}

	// delegate
	public void addScore(int add) {
		model.addScore(add);
	}
	public void launchEvent(BallEvent event) {
		model.notifyObservers(event);
	}

	/**
	 * Called if the shot hit a block
	 */
	public void hit() {
		catched = true;
	}

	@Override
	public void paint(Graphics g) {
		if (catched) return;
		Image image = ImageCenter.getInstance().imgShot;
		g.drawImage(image, getX(), getY(), null);
	}

	public boolean isCatched() {
		return catched;
	}

	public Rectangle getBounds() {
		return new Rectangle(getX(),getY(),width,height);
	}

	public int getX() {
		return Math.round(x);
	}

	public int getY() {
		return Math.round(y);
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}
}
