package ch.lanter.vajanoid.block;

import ch.lanter.vajanoid.ItemType;

/**
 * The BlockFactory provides useful methods to create common types of blocks.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class BlockFactory {

	public static Block createDefaultBlock(int x, int y, BlockImage image) {
		return new RectBlock(x, y, image); 
	}
	
	public static Block createDefaultBlock(int x, int y, BlockImage image, int hp) {
		return new RectBlock(x, y, image, hp); 
	}
	
	public static Block createRockBlock(int x, int y) {
		return BlockFactory.createRockBlock(x, y, BlockImage.SILVER);
	}
	
	public static Block createRockBlock(int x, int y, BlockImage image) {
		Block block = new RectBlock(x, y, image);
		block.setRock(true);
		return block;
	}
	
	public static Block createBlockItemer(int x, int y, BlockImage image, ItemType itemType, int hp, int delay) {
		Block block = new RectBlock(x,y,image,hp);
		ItemerBehavior behavior = new ItemerBehavior(block, itemType,delay);
		block.setBehavior(behavior);
		return block;
	}
	
	public static Block createBlockItemer(int x, int y, BlockImage image, ItemType itemType, int hp) {
		return createBlockItemer(x, y, image, itemType, hp, ItemerBehavior.DEFAULT_DELAY);
	}
	
	public static Block createBlockItemer(int x, int y, BlockImage image, ItemType itemType) {
		return createBlockItemer(x,y,image,itemType,1);
	}
	
	public static Block createRockBlockItemer(int x, int y, BlockImage image, ItemType itemType, int delay) {
		Block block = createBlockItemer(x,y,image,itemType,1,delay);
		block.setRock(true);
		return block;
	}
	
	public static Block createRockBlockItemer(int x, int y, BlockImage image, ItemType itemType) {
		return createRockBlockItemer(x, y, image, itemType, ItemerBehavior.DEFAULT_DELAY);
	}
}
