package ch.lanter.vajanoid.block;

import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URL;

import javax.imageio.ImageIO;

/**
 * The BlockImage contains an image of a block.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public enum BlockImage {

	// Instances of enum BlockImage
	YELLOW		(new Assigner() { BufferedImage getImage() { return block1; }} ),
	ORANGE		(new Assigner() { BufferedImage getImage() { return block2; }} ),
	RED			(new Assigner() { BufferedImage getImage() { return block3; }} ),
	PINK		(new Assigner() { BufferedImage getImage() { return block4; }} ),
	PURPLE		(new Assigner() { BufferedImage getImage() { return block5; }} ),
	BLUE		(new Assigner() { BufferedImage getImage() { return block6; }} ),
	DARK_BLUE	(new Assigner() { BufferedImage getImage() { return block7; }} ),
	GREEN		(new Assigner() { BufferedImage getImage() { return block8; }} ),
	DARK_GREEN	(new Assigner() { BufferedImage getImage() { return block9; }} ),
	BLACK		(new Assigner() { BufferedImage getImage() { return block10; }} ),
	WHITE		(new Assigner() { BufferedImage getImage() { return block11; }} ),
	SILVER		(new Assigner() { BufferedImage getImage() { return block12; }} ),
	IKRK		(new Assigner() { BufferedImage getImage() { return block13; }} );
	
	private static abstract class Assigner { // due to this class, it is possible to change the block's images during runtime
		abstract BufferedImage getImage(); 
	}
	
	private static String imageDir = "/ress/blocks2/"; // default path to the directory
	private static BufferedImage block1;
	private static BufferedImage block2;
	private static BufferedImage block3;
	private static BufferedImage block4;
	private static BufferedImage block5;
	private static BufferedImage block6;
	private static BufferedImage block7;
	private static BufferedImage block8;
	private static BufferedImage block9;
	private static BufferedImage block10;
	private static BufferedImage block11;
	private static BufferedImage block12;
	private static BufferedImage block13;
	
	static {
		loadImagesFromDir(imageDir); // load the block images from the default directory
	}
	
	/**
	 * Load the block Images from the given directory
	 * @param dir -  the directory
	 */
	private static void loadImagesFromDir(String dir) {
		block1 = loadImage(dir+"block_yellow.png");
		block2 = loadImage(dir+"block_orange.png");
		block3 = loadImage(dir+"block_red.png");
		block4 = loadImage(dir+"block_pink.png");
		block5 = loadImage(dir+"block_purple.png");
		block6 = loadImage(dir+"block_blue.png");
		block7 = loadImage(dir+"block_darkblue.png");
		block8 = loadImage(dir+"block_green.png");
		block9 = loadImage(dir+"block_darkgreen.png");
		block10 = loadImage(dir+"block_black.png");
		block11 = loadImage(dir+"block_white.png");
		block12 = loadImage(dir+"block_silver.png");
		block13 = loadImage(dir+"block_ikrk.png");
	}
	
	/**
	 * Load an Image from the given path
	 * @param path - the path
	 * @return - the Image
	 */
	private static BufferedImage loadImage(String path) {
		BufferedImage image = null;
		try {
			
			URL url = BlockImage.class.getResource(path);
			image = ImageIO.read(url);
		} catch (Exception e) {
			System.err.println("Fehler beim Lesen der Bild-Datei "+new File(path).getAbsolutePath());
			e.printStackTrace();
		}
		return image;
	}
	
	private Assigner assigner; // the assigner for a BlockImage
	
	private BlockImage(Assigner assigner) {
		this.assigner = assigner;
	}
	
	public BufferedImage getImage() { // delegate
		return assigner.getImage();
	}
}
