package ch.lanter.vajanoid.block;

import java.awt.Image;

import ch.lanter.vajanoid.Ball;
import ch.lanter.vajanoid.Shot;

/**
 * RectBlock is a subclass of the abstract class Block. It has the extension of
 * health points (hp), which indicated the healthness of a block. However, the
 * fact whether a block is rock or not is to be consider with a higher
 * importance, since the hp of a rock block are absolutely pointless. A hp below
 * zero may indicate, that a block is destroyed but can't be taken as
 * guaranteed. So far, a golden ball is able to destroy every block, no matter
 * whether it is rock or not, but this is open to get changed.
 *
 * @author Martin Lanter, LanterSoft 2010
 */
public class RectBlock extends Block {

	private BlockImage image; // the block's image

	private int inithp; // hp at the beginning
	private int hp; // healt points indicates the healtness but not, whether the block is destroyed or not

	public RectBlock(int x, int y, BlockImage image) {
		this(x,y,image,1);
	}

	public RectBlock(int x, int y, BlockImage image, int hp) {
		super(x,y,32,20);
		this.inithp = hp;
		this.hp = hp;
		this.image = image;
		setBehavior(new DefaultBehavior());
	}

	@Override
	public Image getImage() {
		return image.getImage();
	}

	/**
	 * Called, if the given ball has hit the block
	 */
	@Override
	public void interract(Ball ball) {
		float bx = ball.x;
		float by = ball.y;
		int bw = ball.width;
		int bh = ball.height;
		int x = getBlockX();
		int y = getBlockY();
		int w = getBlockWidth();
		int h = getBlockHeight();
		int diffx = getXMax()-getXMin();
		int diffy = getYMax()-getYMin();
		int xx = getBlockX()+getBlockWidth()/2; // middle of the block
		int yy = getBlockY()+getBlockHeight()/2;
		boolean stroked = false;

		if (ball.vx>0) { // Ball moving to the right
			if ((bx+bw-x+diffx)%diffx<w && (by+bh/2-y+diffy)%diffy<h) {
				stroked = true;
				if (!ball.isInvincible())
					ball.switchVx();
			}
		} else { // Ball moving to the left
			if ((bx-x+diffx)%diffx<w && (by+bh/2-y+diffy)%diffy<h) {
				stroked = true;
				if (!ball.isInvincible())
					ball.switchVx();
			}
		}

		if (ball.vy>0) { // Ball moving to the bottom
			if ((by+bh-y+diffy)%diffy<h && (bx+bw/2-x+diffx)%diffx<w) {
				stroked = true;
				if (!ball.isInvincible())
					ball.switchVy();
			}
		} else { // Ball moving to the top
			if ((by-y+diffy)%diffy<h && (bx+bw/2-x+diffx)%diffx<w) {
				stroked = true;
				if (!ball.isInvincible())
					ball.switchVy();
			}
		}

		if (ball.isInvincible() && stroked) hp = 0; // invicible balls destroy a block immediately
		if (stroked && (!isRock()?--hp:hp)<=0) {
			setDestroyed(true);
			getBehavior().blockDestroyed(xx, yy, inithp, ball);
		} else if (stroked) {
			getBehavior().blockStroked(xx, yy, ball);
		}
	}

	/**
	 * Called, if a shot has hit the block
	 */
	@Override
	public void interract(Shot shot) {
		int sx = shot.getX();
		int sy = shot.getY();
		int sw = shot.getWidth();
		int sh = shot.getHeight();
		int dxm = getXMax()-getXMin();
		int dym = getYMax()-getYMin();
		int xx = getBlockX();
		int yy = getBlockY();
		int ww = getBlockWidth();
		int hh = getBlockHeight();
		if ((sx+sw-xx+dxm)%dxm-sw<ww && (sy+sh-yy+dym)%dym-sh<hh) {
			shot.hit();
			int mx = getBlockX()+getBlockWidth()/2;
			int my = getBlockY()+getBlockHeight()/2;
			if (!isRock() && --hp<=0) {
				setDestroyed(true);
				getBehavior().blockDestroyed(mx, my, inithp, shot);
			} else {
				getBehavior().blockShot(mx, my, shot);
			}
		}
	}

	@Override
	public void setDestroyed(boolean destroyed) {
		if (!destroyed && isDestroyed()) hp = inithp;
		super.setDestroyed(destroyed);
	}

	public int getInitHp() {
		return inithp;
	}
}
