package ch.lanter.vajanoid.block;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;

import javax.swing.JLabel;

import ch.lanter.vajanoid.Ball;
import ch.lanter.vajanoid.Shot;
import ch.lanter.vajanoid.gui.HighlightLabel;

/**
 * A Textblock only shows text but is death in all manner otherwise.
 *
 * @author Martin Lanter, LanterSoft 2010
 */
public class TextBlock extends Block {

	private static final long serialVersionUID = 200911210254L;
	
	private JLabel lbl; // use the label to paint the text
	
	public TextBlock(String text) {
		super(0,0,32,20);
		lbl = new HighlightLabel(text,2,Color.WHITE,Color.BLACK);
		lbl.setFont(new Font(Font.SANS_SERIF,Font.BOLD,16));
		Dimension dim = lbl.getPreferredSize();
		lbl.setSize(dim);
		setBlockWidth(dim.width);
		setBlockHeight(dim.height);
	}
	
	public void setText(String text) {
		int width = getBlockWidth();
		lbl.setText(text);
		Dimension dim = lbl.getPreferredSize();
		lbl.setSize(dim);
		setBlockWidth(dim.width);
		setBlockX(getBlockX()+(width-dim.width)/2);
	}
	
	@Override
	public void paint(Graphics g) {
		if (!isVisible()) return;
		Rectangle clip = g.getClipBounds();
		BufferedImage image = new BufferedImage(clip.width,clip.height, BufferedImage.TYPE_INT_ARGB);
		Graphics2D gi = (Graphics2D) image.createGraphics();
		gi.translate(clip.x,clip.y);
		lbl.paint(gi);
		Graphics2D g2 = (Graphics2D) g.create();
		g2.setComposite(AlphaComposite.SrcOver.derive(getOpacity()));
		g2.drawImage(image,getBlockX(),getBlockY(),null);
		g2.dispose();
		gi.dispose();
	}
	
	public boolean isDestroyed() {
		return true;
	}
	
	@Override
	public Image getImage() { // no image
		return null;
	}

	@Override
	public void interract(Ball ball) { } // do nothing

	@Override
	public void interract(Shot shot) { } // do nothing
}
