package ch.lanter.vajanoid.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JComponent;

import ch.lanter.vajanoid.BallEvent;
import ch.lanter.vajanoid.ImageCenter;
import ch.lanter.vajanoid.Model;
import ch.lanter.vajanoid.BallEvent.TYPE;

/**
 * Displays the number of lives.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class CmpLifes extends JComponent implements Observer {

	private static final long serialVersionUID = 201001130147L;

	private Model model;
	private int brd = 6;
	private int yplus;

	private HighlightLabel lbl;

	public CmpLifes(Model model) {
		this.model = model;
		model.addObserver(this);
		lbl = new HighlightLabel("no data",1,Color.BLACK,new Color(255,255,255,64));
		lbl.setFont(new Font(Font.SANS_SERIF,Font.BOLD,12));
		yplus = lbl.getPreferredSize().height+brd-4;
		add(lbl);
	}

	@Override
	public void paintComponent(Graphics g) {
		int w = getWidth();
		int h = getHeight();
		int lifes = model.getLives();
		Image image = ImageCenter.getInstance().imgBall;

		// correct label
		lbl.setText( (lifes>=0? lifes :0) +" balls left");
		Dimension dim = lbl.getPreferredSize();
		lbl.setLocation((w-dim.width)/2+1,brd);
		lbl.setSize(dim);

		// draw pane and border
		g.setColor(new Color(200,200,255));
		g.fillRoundRect(brd, brd+yplus, w-2*brd+2, h-2*brd+2-yplus-brd, 5, 5);
		g.setColor(new Color(100,100,255));
		g.drawRoundRect(brd, brd+yplus, w-2*brd+2, h-2*brd+2-yplus-brd, 5, 5);
		g.setColor(new Color(255,255,255));
		g.drawRoundRect(brd-1, brd-1+yplus, w-2*brd+2, h-2*brd+2-yplus-brd, 5, 5);

		// draw balls
		if (lifes>=1) g.drawImage(image,brd+4	,brd+4+yplus	,null);
		if (lifes>=2) g.drawImage(image,brd+4+19,brd+4+yplus	,null);
		if (lifes>=3) g.drawImage(image,brd+4+38,brd+4+yplus	,null);
		if (lifes>=4) g.drawImage(image,brd+4	,brd+4+19+yplus,null);
		if (lifes>=5) g.drawImage(image,brd+4+19,brd+4+19+yplus,null);
		if (lifes>=6) g.drawImage(image,brd+4+38,brd+4+19+yplus,null);
	}

	public void update(Observable o, Object arg) {
		BallEvent event = (BallEvent) arg;
		if (event.getType()==TYPE.LIFE_GAINED ||
				event.getType()==TYPE.LIFE_LOST ||
				event.getType()==TYPE.RESTERT) {
			repaint();
		}
	}

	@Override
	public Dimension getPreferredSize() {
		return new Dimension(72,54+yplus+brd);
	}

	@Override
	public Dimension getMaximumSize() {
		return this.getPreferredSize();
	}
}