package ch.lanter.vajanoid.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JLabel;

/**
 * A text component that highlights the foreground with a color in the
 * background.
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class HighlightLabel extends JLabel {

	private static final long serialVersionUID = 201001130148L;

	private int strength;
	private Color frontColor;
	private Color glossColor;

	public HighlightLabel(String text, int strength) {
		this(text,strength,Color.WHITE,Color.BLACK);
	}

	public HighlightLabel(String text, int strength, Color frontColor, Color glossColor) {
		super(text);
		this.strength = strength;
		this.frontColor = frontColor;
		this.glossColor = glossColor;
	}

	/**
	 * Needs to be overriden
	 */
	@Override
	public Dimension getPreferredSize() {
		Dimension dim = super.getPreferredSize();
		return new Dimension(dim.width+2*strength, dim.height+2*strength);
	}

	/**
	 * Needs to be overriden
	 */
	@Override
	public Dimension getMinimumSize() {
		Dimension dim = super.getPreferredSize();
		return new Dimension(dim.width+2*strength, dim.height+2*strength);
	}

	/**
	 * Needs to be overriden (for BoxLayout !!!)
	 */
	@Override
	public Dimension getMaximumSize() {
		Dimension dim = super.getMaximumSize();
		return new Dimension(dim.width+2*strength, dim.height+2*strength);
	}

	public int getStrength() {
		return strength;
	}

	public void setStrngth(int strength) {
		this.strength = strength;
	}

	@Override
	public void paintComponent(Graphics g) {
		String title = getText();
		Graphics2D g2 = (Graphics2D) g.create();

        FontMetrics metrics = g2.getFontMetrics();
        double ty = (metrics.getHeight() + metrics.getAscent()) / 2 - strength;
        g2.translate(0, ty);

		Composite oldComposite = g2.getComposite();

		drawTextHilghlight(g2,title,strength,0.8f);

		g2.setComposite(oldComposite);
		g2.setColor(frontColor);
		g2.drawString(title,strength,strength);

		g2.translate(0,-ty);
	}

	private void drawTextHilghlight(Graphics2D g2, String title, int size, float opacity) {
		g2.setColor(glossColor);
		for (int i=-size;i<=size;i++) {
			for (int j=-size;j<=size;j++) {
				double distance = (i*i+j*j)/4; // must be 4 (int), not 4d
				float alpha = opacity;
				if (distance>0.0d) {
					alpha = (float) (1.0f / ((distance*size)*opacity));
				}
				g2.setComposite(AlphaComposite.SrcOver.derive(alpha));
				g2.drawString(title,i+size,j+size);
			}
		}
	}
}
