package ch.lanter.vajanoid.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.Paint;
import java.awt.RadialGradientPaint;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import javax.swing.JButton;

/**
 * The buttons in the game
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class JVButton extends JButton {

	private static final long serialVersionUID = 201001130149L;

	public JVButton(String str) {
		super(str);
		setBorderPainted(false);
		setContentAreaFilled(false);
	}

	@Override
	public void paint(Graphics g) {
		int w = this.getWidth();
		int h = this.getHeight();

		BufferedImage image = new BufferedImage(w,h,BufferedImage.TYPE_INT_ARGB);
		Graphics2D g2 = (Graphics2D) image.getGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setClip(g.getClip());
		g2.setColor(Color.RED);
		g2.fillRoundRect(3,0,w-5,h-3,8,8); // visible pixels
		g2.setComposite(AlphaComposite.SrcIn);

		BufferedImage superImage = new BufferedImage(w,h,BufferedImage.TYPE_INT_ARGB);
		super.paint(superImage.getGraphics());
		g2.drawImage(superImage,0,0,null);

		Graphics2D gg = (Graphics2D) g.create();
		gg.drawImage(image,0,0,null);
		gg.setColor(new Color(100,100,255,64));
		gg.drawRoundRect(3,0,w-5,h-3,8,8); // border
	}

	@Override
	public void paintComponent(Graphics g) {
		String title = getText();
		Graphics2D g2 = (Graphics2D) g.create();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
		Paint p;
        if (!getModel().isPressed()) {
        	 p = new LinearGradientPaint(0.0f, 0.0f, 0.0f, getHeight(),
     				new float[] { 0.0f, 0.499f, 0.5f, 1.0f },
     				new Color[] { 	new Color(120,130,255),
     								new Color(100,100,255),
     								new Color(60,60,255),
     								new Color(120,130,255).darker() });
        } else {
        	 p = new LinearGradientPaint(0.0f, 0.0f, 0.0f, getHeight(),
     				new float[] { 0.0f, 0.499f, 0.5f, 1.0f },
     				new Color[] { 	new Color(90,100,255),
     								new Color(70,70,255),
     								new Color(30,30,200),
     								new Color(90,100,255).darker() });
        }
		g2.setPaint(p);
		g2.fillRect(0, 0, getWidth(), getHeight());

		if (getModel().isRollover()) {
			p = new RadialGradientPaint(
					getWidth()/2,
					getHeight()/4+50,
					70,
					new float[] {0f, 1f},
					new Color[] {new Color(180,180,255,200),new Color(180,180,255,0) }
					);
			g2.setPaint(p);
			g2.fillRect(0,0,getWidth(),getHeight());
		}

		// write text
		int strength = 1;
		float opacity = getModel().isRollover()? .9f :0.5f;
		int tx = (getWidth()-g2.getFontMetrics().stringWidth(title))/2-strength;
		int ty = g2.getFontMetrics().getHeight()-strength;
		if (getModel().isRollover() && !getModel().isPressed()) ty = ty-1;
		g2.translate(tx,ty);
		Composite oldComposite = g2.getComposite();
		drawTextHilghlight(g2,title,strength,opacity);

		g2.setComposite(oldComposite);
		g2.setColor(new Color(0,0,0));
		g2.drawString(title,strength,strength);

		g2.translate(-tx,-ty);
		g2.dispose();
	}

	private void drawTextHilghlight(Graphics2D g2, String title, int size, float opacity) {
		g2.setColor(new Color(255,255,255,200));
		for (int i=-size;i<=size;i++) {
			for (int j=-size;j<=size;j++) {
				double distance = (i*i+j*j)/4;
				float alpha = opacity;
				if (distance>0.0d) {
					alpha = (float) (1.0f / ((distance*size)*opacity));
				}
				g2.setComposite(AlphaComposite.SrcOver.derive(alpha));
				g2.drawString(title,i+size,j+size);
			}
		}
	}
}
