package ch.lanter.vajanoid.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import ch.lanter.vajanoid.Command;
import ch.lanter.vajanoid.Model;

/**
 * The sidebar of the game
 * 
 * @author Martin Lanter, LanterSoft 2010
 */
public class PnlSidebar extends JPanel {

	private static final long serialVersionUID = 201001130151L;

	public PnlSidebar(final Model model, final ActionListener al) {

		CmpLifes cmpLifes = new CmpLifes(model);
		CmpCheat cmpCheat = new CmpCheat(model,al);
		CmpLevel cmpLevel = new CmpLevel(model);
		CmpScore cmpScore = new CmpScore(model);

		JPanel pnlButtons = new JPanel(new GridLayout(3,1));
		pnlButtons.setBorder(new EmptyBorder(0,0,10,0));
		pnlButtons.setOpaque(false);
		JButton btnStart = new JVButton("Start");
		JButton btnPause = new JVButton("Pause");
		JButton btnUnlock = new JVButton("Unlock");
		btnUnlock.setToolTipText("Push all balls randomly");
		btnStart.addActionListener(al);
		btnPause.addActionListener(al);
		btnUnlock.addActionListener(al);
		btnStart.setActionCommand(Command.START);
		btnPause.setActionCommand(Command.PAUSE);
		btnUnlock.setActionCommand(Command.UNLOCK);
		btnStart.setFocusable(false);
		btnPause.setFocusable(false);
		btnUnlock.setFocusable(false);

		pnlButtons.add(btnStart);
		pnlButtons.add(btnPause);
		pnlButtons.add(btnUnlock);
		pnlButtons.setMaximumSize(pnlButtons.getPreferredSize());

		JPanel pnlOptions = new JPanel();
		pnlOptions.setLayout(new BoxLayout(pnlOptions, BoxLayout.X_AXIS));
		pnlOptions.setOpaque(false);
		final JCheckBox cbxMouse = new JCheckBox();
		cbxMouse.setFocusable(false);
		cbxMouse.setSelected(model.isShowCursor());
		cbxMouse.setOpaque(false);
		JLabel lblMouse = new HighlightLabel("Cursor",1,Color.BLACK,new Color(255,255,255,64));
		lblMouse.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent event) {
				boolean b = !cbxMouse.isSelected();
				cbxMouse.setSelected(b);
				model.setShowCursor(b);
			}
		});
		cbxMouse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				model.setShowCursor(cbxMouse.isSelected());
			}
		});
		pnlOptions.add(cbxMouse);
		pnlOptions.add(lblMouse);
		pnlOptions.setAlignmentX(CENTER_ALIGNMENT);
		
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		add(cmpLifes);
		add(pnlButtons);
		add(pnlOptions);
		add(cmpScore);
		add(Box.createVerticalGlue());
		add(cmpLevel);
		add(cmpCheat);
		setBorder(new EmptyBorder(0,6,10,6));

		Dimension dim = getPreferredSize();
		setMaximumSize(new Dimension(dim.width,model.getHeight()));
	}

	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g.create();
		GradientPaint gradint = new GradientPaint(new Point(getWidth(),0),new Color(170,180,255),new Point(0,0),new Color(150,150,255));
		g2.setPaint(gradint);
		Rectangle clip = g.getClipBounds();
		g2.fillRect(clip.x, clip.y, clip.width, clip.height);
		g2.setColor(new Color(200,200,255));
		g2.drawLine(0,0,0,getHeight());
		g2.dispose();
	}
}
