package ch.lanter.vajanoid.level;

import java.awt.Point;

import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockImage;
import ch.lanter.vajanoid.block.Path;
import ch.lanter.vajanoid.block.RectBlock;
import ch.lanter.vajanoid.path.PolygonalPath;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level02 extends AbstractLevel {

	// blocks to be painted
	private Block[] blocks;
	
	// paths that define the block's positions at every time
	private PolygonalPath[] paths;
	
	public Level02() {
		int i = 0;
		blocks = new Block[68]; // 68 blocks in this level
		paths = new PolygonalPath[3]; // 3 PolygonalPaths in this level

		paths[0] = new PolygonalPath(120f); // this path has a speed of 120 pixel per second (clockwise)
		paths[0].addVertice(new Point(32,20)); // define the vertices of this path
		paths[0].addVertice(new Point(320,20));
		paths[0].addVertice(new Point(320,240));
		paths[0].addVertice(new Point(32,240));

		paths[1] = new PolygonalPath(-120f); // this path has a speed of 120 pixel per second (anticlockwise)
		paths[1].addVertice(new Point(96,60));
		paths[1].addVertice(new Point(256,60));
		paths[1].addVertice(new Point(256,200));
		paths[1].addVertice(new Point(96,200));

		paths[2] = new PolygonalPath(120f);
		paths[2].addVertice(new Point(160,120));
		paths[2].addVertice(new Point(192,120));
		paths[2].addVertice(new Point(192,140));
		paths[2].addVertice(new Point(160,140));
		
		for (int j=0;j<40;j++) // the outmost square
			blocks[i++] = createBlock(0,0,BlockImage.DARK_GREEN, paths[0]);

		for (int j=0;j<24;j++) // the middle square
			blocks[i++] = createBlock(0,0,BlockImage.DARK_BLUE, paths[1]);
		
		for (int j=0;j<4;j++) // the inner square
			blocks[i++] = createBlock(0,0,BlockImage.PINK, paths[2]);
		
	}
	
	/**
	 * Create a Block at (x,y) with the given image and add it to the given path
	 */
	private Block createBlock(int x, int y, BlockImage image, PolygonalPath path) {
		RectBlock ret = new RectBlock(x, y, image);
		path.addBlock(ret);
		return ret;
	}
	
	@Override
	public void start() {
		for (PolygonalPath p:paths) {
			p.arrange(); // the PolygonalPath arranges its blocks along the route they are sliding on
		}
		super.start();
	}

	@Override
	public Block[] getBlocks() {
		return blocks;
	}
	
	@Override
	public Path[] getPaths() {
		return paths;
	}

	@Override
	public String getName() {
		return "Square";
	}

}
