package ch.lanter.vajanoid.level;

import java.util.Observable;

import ch.lanter.vajanoid.BallEvent;
import ch.lanter.vajanoid.BallEvent.TYPE;
import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockFactory;
import ch.lanter.vajanoid.block.BlockImage;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level15 extends AbstractLevel {

	private Block[] blocks1;
	private Block[] blocks2;
	private Block[] blocks;
	
	private boolean one; // true, if blocks1 is visible
	private boolean isStarted;
	
	public Level15() {
		this.one = true;
		this.isStarted = false;
		
		BlockImage[] imgs = new BlockImage[] {BlockImage.YELLOW, BlockImage.ORANGE, BlockImage.RED, BlockImage.PINK};
		blocks1 = new Block[36];
		for (int i=0;i<9;i++) {
			blocks1[i*4+0] = BlockFactory.createDefaultBlock(32,40+i*20,imgs[(i+0)%4]);
			blocks1[i*4+1] = BlockFactory.createDefaultBlock(64,40+i*20,imgs[(i+1)%4]);
			blocks1[i*4+2] = BlockFactory.createDefaultBlock(96,40+i*20,imgs[(i+2)%4]);
			blocks1[i*4+3] = BlockFactory.createDefaultBlock(128,40+i*20,imgs[(i+3)%4]);
		}
		
		imgs = new BlockImage[] {BlockImage.PURPLE, BlockImage.BLUE, BlockImage.DARK_GREEN, BlockImage.GREEN};
		blocks2 = new Block[36];
		for (int i=0;i<9;i++) {
			blocks2[i*4+0] = BlockFactory.createDefaultBlock(224,40+i*20,imgs[(i+0)%4]);
			blocks2[i*4+1] = BlockFactory.createDefaultBlock(256,40+i*20,imgs[(i+1)%4]);
			blocks2[i*4+2] = BlockFactory.createDefaultBlock(288,40+i*20,imgs[(i+2)%4]);
			blocks2[i*4+3] = BlockFactory.createDefaultBlock(320,40+i*20,imgs[(i+3)%4]);
		}
		
		blocks = new Block[blocks1.length+blocks2.length];
		System.arraycopy(blocks1, 0, blocks, 0, blocks1.length);
		System.arraycopy(blocks2, 0, blocks, blocks1.length, blocks2.length);
	}
	
	@Override
	public String getName() {
		return "Blocky";
	}
	
	@Override
	public void update(Observable o, Object arg) {
		if (!this.isStarted) return; 
		
		BallEvent event = (BallEvent) arg;
		if (event.getType()==TYPE.BALL_PADDED) {
			if (one) {
				for (Block b:blocks1) b.setActive(true);
				for (Block b:blocks2) b.setActive(false);
			} else {
				for (Block b:blocks1) b.setActive(false);
				for (Block b:blocks2) b.setActive(true);
			}
			one = !one;
		}
	}
	
	@Override
	public Block[] getBlocks() {
		return blocks;
	}
	
	@Override
	public boolean isWon() {
		for (Block b:blocks) {
			if (b!=null && b.isAlive()) { // there is any block still alive
				return false;
			}
		}
		return true;
	}
	
	@Override
	public void start() {
		super.start();
		isStarted = true;
	}
	
	@Override
	public void stop() {
		isStarted = false;
	}
	
}
