package ch.lanter.vajanoid.level;

import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockImage;
import ch.lanter.vajanoid.block.RectBlock;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level20 extends AbstractLevel {

	private RectBlock[] blocks;
	private RectBlock[] ikrkL;
	private RectBlock[] ikrkR;
	
	private float v = 80f;
	
	public Level20() {
		int i = 0;
		blocks = new RectBlock[120];
		ikrkL = new RectBlock[12];
		ikrkR = new RectBlock[12];

		/*
		 * 2 = red, 4 = white
		 * 0 = destroyed, 1 healing
		 */
		int s = 3;
		int[][] field = new int[][] { // 12x8
				{ 1, 1, 1, 1, 1, 1, 1, 1},
				{ 1, 1, 1, 1, 1, 1, 1, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, s, s, s, s, s, s, 1},
				{ 1, s, s, s, s, s, s, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, 1, 1, s, s, 1, 1, 1},
				{ 1, 1, 1, 1, 1, 1, 1, 1},
				{ 1, 1, 1, 1, 1, 1, 1, 1} };

		for (int k=0;k<12;k++) {
			for (int j=0;j<8;j++) {
				if (field[k][j]==1) 
					blocks[i++] = new RectBlock(64+j*32,20+k*20,BlockImage.RED);
				else blocks[i++] = new RectBlock(64+j*32,20+k*20,BlockImage.WHITE,2);
			}
		}
		for (int j=0;j<12;j++) {
			ikrkL[j] = createIkrkBlock(-32,20+j*20);
			blocks[i++] = ikrkL[j];
		}
		
		for (int j=0;j<12;j++) {
			ikrkR[j] = createIkrkBlock(384,20+j*20);
			blocks[i++] = ikrkR[j];
		}
	}
	
	@Override
	public void step(long dt) {
		float dx = v*dt/1000f;
		
		// Left
		for (int stage=0;stage<12;stage++) {
			if (ikrkL[stage].isDestroyed()) continue;
			if (stateL[stage]==0) {
				for (int i=0;i<4 && blocks[stage*8+i].isDestroyed();i++) {
					helpL[stage] = blocks[stage*8+i];
					posL[stage] = ikrkL[stage].getBlockX();
					stateL[stage] = 1;
				}
			} else if (stateL[stage]==1) { // send help
				posL[stage] += dx;
				if (posL[stage]>helpL[stage].getBlockX()-helpL[stage].getBlockWidth()) {
					ikrkL[stage].setBlockX(helpL[stage].getBlockX()-helpL[stage].getBlockWidth());
					helpL[stage].setVisible(true);
					helpL[stage].setOpacity(0f);
					progressL[stage] = 0;
					stateL[stage] = 2;
				} else {
					ikrkL[stage].setBlockX(Math.round(posL[stage]));
				}
				
			} else if (stateL[stage]==2) { //heal
				int hp = helpL[stage].getInitHp();
				progressL[stage] += dt;
				helpL[stage].setOpacity((float)progressL[stage]/2000*hp);
				if (progressL[stage]>=2000*hp) {
					helpL[stage].setDestroyed(false);
					helpL[stage].setVisible(true);
					posL[stage] = ikrkL[stage].getBlockX();
					stateL[stage] = 3;
				}
				
			} else if (stateL[stage]==3) { // go back
				posL[stage] -= dx;
				if (posL[stage] < -helpL[stage].getBlockWidth()) {
					ikrkL[stage].setBlockX(-helpL[stage].getBlockWidth());
					stateL[stage] = 0;
				} else {
					ikrkL[stage].setBlockX(Math.round(posL[stage]));
				}
			}
		} // for all
		
		// Right
		for (int stage=0;stage<12;stage++) {
			if (ikrkR[stage].isDestroyed()) continue;
			if (stateR[stage]==0) {
				for (int i=0;i<4 && blocks[stage*8+7-i].isDestroyed();i++) {
					helpR[stage] = blocks[stage*8+7-i];
					posR[stage] = ikrkR[stage].getBlockX();
					stateR[stage] = 1;
				}
			} else if (stateR[stage]==1) { // send help
				posR[stage] -= dx;
				if (posR[stage]<helpR[stage].getBlockX()+helpR[stage].getBlockWidth()) {
					ikrkR[stage].setBlockX(helpR[stage].getBlockX()+helpR[stage].getBlockWidth());
					helpR[stage].setVisible(true);
					helpR[stage].setOpacity(0f);
					progressR[stage] = 0;
					stateR[stage] = 2;
				} else {
					ikrkR[stage].setBlockX(Math.round(posR[stage]));
				}
				
			} else if (stateR[stage]==2) { //heal
				int hp = helpR[stage].getInitHp();
				progressR[stage] += dt;
				helpR[stage].setOpacity((float)progressR[stage]/2000*hp);
				if (progressR[stage]>=2000*hp) {
					helpR[stage].setDestroyed(false);
					helpR[stage].setVisible(true);
					posR[stage] = ikrkR[stage].getBlockX();
					stateR[stage] = 3;
				}
				
			} else if (stateR[stage]==3) { // go back
				posR[stage] += dx;
				if (posR[stage] > 384) {
					ikrkR[stage].setBlockX(384);
					stateR[stage] = 0;
				} else {
					ikrkR[stage].setBlockX(Math.round(posR[stage]));
				}
			}
		} // for all
	}

	RectBlock[] helpL = new RectBlock[12];
	float[] posL = new float[12];
	long[] progressL = new long[12];
	int[] stateL = new int[12]; // 0 = nix, 1 = go to help, 2 = heal, 3 = go back
	
	RectBlock[] helpR = new RectBlock[12];
	float[] posR = new float[12];
	long[] progressR = new long[12];
	int[] stateR = new int[12]; // 0 = nix, 1 = go to help, 2 = heal, 3 = go back
	
	private RectBlock createIkrkBlock(int x, int y) {
		RectBlock block = new RectBlock(x,y,BlockImage.IKRK,5);
		block.setRock(true);
		block.setXMin(-32);
		block.setXMax(416);
		block.setBlockX(x);
		return block;
	}
	
	@Override
	public void start() {
		super.start();
	}

	@Override
	public Block[] getBlocks() {
		return blocks;
	}
	
	@Override
	public String getName() {
		return "Switzerland";
	}
}
