package ch.lanter.vajanoid.level;

import java.awt.Point;

import ch.lanter.vajanoid.Ball;
import ch.lanter.vajanoid.Eventable;
import ch.lanter.vajanoid.Model;
import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockFactory;
import ch.lanter.vajanoid.block.BlockImage;
import ch.lanter.vajanoid.block.DefaultBehavior;
import ch.lanter.vajanoid.block.Path;
import ch.lanter.vajanoid.block.RectBlock;
import ch.lanter.vajanoid.path.PolygonalPath;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level24 extends AbstractLevel {

	private enum STATE {CLOSED, OPENING, OPEN, CLOSING}
	
	private Model model;
	private Block[] blocks;
	private Block[] greens;
	private Block[] buttons;
	private PolygonalPath path;
	
	private STATE state = STATE.CLOSED;
	private float pos = 0; // CLOSED=0, OPEN=1, in the middle=.5
	private float speed = 4f; // pos per second
	private boolean hasBennInside = false;
	
	public Level24(Model model) {
		this.model = model;
		int i = 0;
		blocks = new Block[73];
		greens = new Block[4];
		buttons = new Block[3];
		path = new PolygonalPath(20f);
		
		for (int j=0;j<4;j++) {
			blocks[i++] = BlockFactory.createRockBlock(2*j*32,220);
			greens[j] = BlockFactory.createRockBlock((2*j+1)*32,220,BlockImage.DARK_GREEN);
			blocks[i++] = greens[j];
		}

		blocks[i++] = new RectBlock(288,220,BlockImage.BLACK,3);
		blocks[i++] = new RectBlock(352,220,BlockImage.BLACK,3);
		
		for (int j=0;j<12;j++)
			blocks[i++] = BlockFactory.createRockBlock(256,j*20);

		blocks[i++] = buttons[0] = BlockFactory.createRockBlock(320,60,BlockImage.RED);
		buttons[0].setBehavior(new DoorBehaviour());
		blocks[i++] = buttons[1] = BlockFactory.createRockBlock(320,140,BlockImage.RED);
		buttons[1].setBehavior(new DoorBehaviour());
		blocks[i++] = buttons[2] = BlockFactory.createRockBlock(320,220,BlockImage.RED);
		buttons[2].setBehavior(new DoorBehaviour());
		
		path.addVertice(new Point(1*32,1*20));
		path.addVertice(new Point(2*32,1*20));
		path.addVertice(new Point(2*32,2*20));
		path.addVertice(new Point(3*32,2*20));
		path.addVertice(new Point(3*32,1*20));
		path.addVertice(new Point(4*32,1*20));
		path.addVertice(new Point(4*32,2*20));
		path.addVertice(new Point(5*32,2*20));
		path.addVertice(new Point(5*32,1*20));
		path.addVertice(new Point(6*32,1*20));

		path.addVertice(new Point(6*32,3*20));
		path.addVertice(new Point(4*32,3*20));
		path.addVertice(new Point(4*32,5*20));
		path.addVertice(new Point(5*32,5*20));
		path.addVertice(new Point(5*32,4*20));
		path.addVertice(new Point(6*32,4*20));
		path.addVertice(new Point(6*32,6*20));
		path.addVertice(new Point(5*32,6*20));
		path.addVertice(new Point(5*32,7*20));
		path.addVertice(new Point(6*32,7*20));
		path.addVertice(new Point(6*32,8*20));
		path.addVertice(new Point(4*32,8*20));
		path.addVertice(new Point(4*32,6*20));

		path.addVertice(new Point(3*32,6*20));
		path.addVertice(new Point(3*32,8*20));
		path.addVertice(new Point(1*32,8*20));
		path.addVertice(new Point(1*32,7*20));
		path.addVertice(new Point(2*32,7*20));
		path.addVertice(new Point(2*32,6*20));
		path.addVertice(new Point(1*32,6*20));
		path.addVertice(new Point(1*32,4*20));
		path.addVertice(new Point(2*32,4*20));
		path.addVertice(new Point(2*32,5*20));
		path.addVertice(new Point(3*32,5*20));
		path.addVertice(new Point(3*32,3*20));
		path.addVertice(new Point(1*32,3*20));
		
		for (int j=0;j<24;j++) {
			blocks[i++] = createBlock(BlockImage.DARK_BLUE);
			blocks[i++] = createBlock(BlockImage.BLUE);
		}
	}
	
	private Block createBlock(BlockImage image) {
		Block block = new RectBlock(0,0,image);
		path.addBlock(block);
		return block;
	}
	
	@Override
	public void start() {
		path.arrange();
		super.start();
	}
	
	@Override
	public void step(long dt) {
		super.step(dt);
		checkCloseAndOpen();
		if (state==STATE.OPEN || state==STATE.CLOSED) {
			return;
		} else if (state==STATE.OPENING) {
			float dpos = dt*speed/1000;
			if (pos+dpos>=1) {
				pos = 1;
				state = STATE.OPEN;
			} else {
				pos += dpos;
			}
		} else if (state==STATE.CLOSING) {
			float dpos = dt*speed/1000;
			if (pos-dpos<=0) {
				pos = 0;
				state = STATE.CLOSED;
			} else {
				pos -= dpos;
			}
		}
		int xx=0, yy=20;
		if (pos>=.5f) {
			xx = Math.round(2*(pos-.5f)*32);
		} else {
			yy = Math.round((2*pos)*20);
		}
		for (int i=0;i<greens.length;i++) {
			greens[i].setBlockX((2*i+1)*32-xx);
			greens[i].setBlockY(220-yy);
		}
	}
	
	private void checkCloseAndOpen() {
		if (buttons[0].isDestroyed() &&
				buttons[1].isDestroyed() &&
				buttons[2].isDestroyed()) {
			for (Block b:buttons) {
				b.setDestroyed(false);
//				b.setRock(true);
				b.setVisible(true);
			}
		}
		if (state==STATE.OPEN || state==STATE.OPENING) {
			Ball[] balls = model.getBalls();
			for (Ball ball:balls) {
				if (ball.x<256 && ball.y<220) {
					hasBennInside = true;
					return;
				}
			}
			// No ball in the top left rectangle
			if (hasBennInside) {
				state = STATE.CLOSING;
				hasBennInside = false;
			}
		} else if (state==STATE.CLOSED || state==STATE.CLOSING) {
			int count = 0;
			Ball[] balls = model.getBalls();
			for (Ball ball:balls) {
				if (ball.x<256 && ball.y<220) {
					count++;
				}
			}
			if (count==model.getBallCount()) { // all living balls in the top left rectangle
				state = STATE.OPENING;
			}
		}
		return;
	}

	@Override
	public Block[] getBlocks() {
		return blocks;
	}
	
	@Override
	public Path[] getPaths() {
		return new Path[] {path};
	}

	@Override
	public String getName() {
		return "Button Anger";
	}
	
	private class DoorBehaviour extends DefaultBehavior {
		public void blockStroked(int x, int y, Eventable eventable) {
			if (state==STATE.CLOSED || state==STATE.CLOSING) { // closed or closing
				state = STATE.OPENING;
			} else {
				state = STATE.CLOSING;
			}
		}
		public void blockDestroyed(int x, int y, int hp, Eventable eventable) {
			blockStroked(x,y,eventable);
		}
	}
}
