package ch.lanter.vajanoid.level;

import java.awt.Point;
import java.util.Random;

import ch.lanter.vajanoid.Eventable;
import ch.lanter.vajanoid.ItemReleasedEvent;
import ch.lanter.vajanoid.ItemType;
import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockFactory;
import ch.lanter.vajanoid.block.BlockImage;
import ch.lanter.vajanoid.block.DefaultBehavior;
import ch.lanter.vajanoid.block.Path;
import ch.lanter.vajanoid.block.RectBlock;
import ch.lanter.vajanoid.path.PolygonalPath;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level28 extends AbstractLevel {

	private Block[] blocks;
	private Block[] purples;
	private Block goldBlock;
	private PolygonalPath path;
	
	public Level28() {
		int i = 0;
		blocks = new Block[56];
		path = new PolygonalPath(120f);
		path.addVertice(new Point(3*32,100));
		path.addVertice(new Point(8*32,100));
		path.addVertice(new Point(8*32,220));
		path.addVertice(new Point(3*32,220));

		int k = 0;
		purples = new Block[10];
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(0*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(1*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(2*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(4*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(5*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(6*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(7*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(9*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(10*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		blocks[i++] = purples[k++] = BlockFactory.createBlockItemer(11*32,180,BlockImage.PURPLE,ItemType.PURPLE);
		
		int delay = 1000;
		blocks[i++] = BlockFactory.createRockBlockItemer(0*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(1*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(2*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(4*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(5*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(6*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(7*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(9*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(10*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		blocks[i++] = BlockFactory.createRockBlockItemer(11*32,200,BlockImage.YELLOW,ItemType.YELLOW,delay);
		
		for (int j=0;j<11;j++)
			blocks[i++] = createMovingBlock();
		
		for (int j=8;j>=1;j--)
			blocks[i++] = new RectBlock(32,j*20,BlockImage.DARK_GREEN);
		
		for (int j=2;j<10;j++)
			blocks[i++] = new RectBlock(j*32,20,BlockImage.DARK_GREEN);
		
		for (int j=1;j<=8;j++)
			blocks[i++] = new RectBlock(320,j*20,BlockImage.DARK_GREEN);
		
		goldBlock = BlockFactory.createRockBlockItemer(176,140,BlockImage.ORANGE,ItemType.ORANGE);
		goldBlock.setActive(false);
		blocks[i++] = goldBlock;
	}
	
	private Block createMovingBlock() {
		Block block = BlockFactory.createRockBlock(0,0);
		block.setBehavior(new HitBehaviour());
		path.addBlock(block);
		return block;
	}
	
	@Override
	public void start() {
		path.arrange();
		super.start();
	}

	@Override
	public Block[] getBlocks() {
		return blocks;
	}
	
	@Override
	public Path[] getPaths() {
		return new Path[] {path};
	}

	@Override
	public String getName() {
		return "Elevator";
	}
	
	private class HitBehaviour extends DefaultBehavior {
		private Random rand = new Random();
		public void blockStroked(int x, int y, Eventable eventable) {
			int count = 0;
			for (int i=0; i<purples.length;i++) {
				count += purples[i].isDestroyed() ?0 :1;
			}
			if (count==0) return;
			else if (count<=2) goldBlock.setActive(true);
			int r = 0;
			do {
				r = rand.nextInt(purples.length);
			} while(purples[r].isDestroyed());
			int xx = purples[r].getBlockX()+purples[r].getBlockWidth()/2;
			int yy = purples[r].getBlockY()+purples[r].getBlockHeight()/2;
			eventable.launchEvent(new ItemReleasedEvent(xx,yy,ItemType.PURPLE));
		}
	}

}
