package ch.lanter.vajanoid.level;

import java.awt.Point;
import java.util.Observable;
import java.util.Observer;
import java.util.Random;

import ch.lanter.vajanoid.BallEvent;
import ch.lanter.vajanoid.Eventable;
import ch.lanter.vajanoid.ItemReleasedEvent;
import ch.lanter.vajanoid.ItemType;
import ch.lanter.vajanoid.Model;
import ch.lanter.vajanoid.BallEvent.TYPE;
import ch.lanter.vajanoid.block.Block;
import ch.lanter.vajanoid.block.BlockImage;
import ch.lanter.vajanoid.block.DefaultBehavior;
import ch.lanter.vajanoid.block.EmptyBehavior;
import ch.lanter.vajanoid.block.Path;
import ch.lanter.vajanoid.block.RectBlock;
import ch.lanter.vajanoid.block.TextBlock;
import ch.lanter.vajanoid.path.FleePath;
import ch.lanter.vajanoid.path.ShiftPath;
import ch.lanter.vajanoid.path.TriggeredPath;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class Level30 extends AbstractLevel {

	private Model model;
	
	private Block[] blocks;
	private Block[] face;
	private Block[] itemBlocks;
	private TextBlock textBlock;
	private Path[] paths;
	private TriggeredPath[] triggers;
	private TriggeredPath[] angryTriggers;
	private ItemType droppingItem;
	
	private long time;
	private boolean faceShow = false;
	private boolean angry = false;
	private boolean texted = false;
	private boolean texted2 = false;
	
	
	public Level30(Model model) {
		this.model = model;
		this.droppingItem = ItemType.BLUE;
		int i = 0;
		blocks = new Block[99];
		
		ShiftPath up1 = new ShiftPath(100f);
		ShiftPath up2 = new ShiftPath(200f);
		ShiftPath up3 = new ShiftPath(300f);
		ShiftPath dw1 = new ShiftPath(100f);
		ShiftPath lf1 = new ShiftPath(200f);
		ShiftPath lf2 = new ShiftPath(200f);
		ShiftPath rt1 = new ShiftPath(200f);
		ShiftPath rt2 = new ShiftPath(200f);

		ShiftPath angrydw = new ShiftPath(20f);
		ShiftPath angryup = new ShiftPath(20f);

		up1.addVector(new Point(0,-20));
		up2.addVector(new Point(0,-20));
		up3.addVector(new Point(0,-40));
		dw1.addVector(new Point(0,20));
		lf1.addVector(new Point(32,0));
		lf1.addVector(new Point(0,-20));
		lf2.addVector(new Point(64,0));
		rt1.addVector(new Point(-32,0));
		rt1.addVector(new Point(0,-20));
		rt2.addVector(new Point(-64,0));

		angrydw.addVector(new Point(0,20));
		angryup.addVector(new Point(0,-20));

		Path fleePath = new FleePath(model,6f);
		
		triggers = new TriggeredPath[] {
				up1,up2,up3,dw1,lf1,lf2,rt1,rt2
		};
		
		angryTriggers = new TriggeredPath[] {
				angrydw,angryup
		};
		
		paths = new Path[] {
				up1,up2,up3,dw1,lf1,lf2,rt1,rt2,angrydw,angryup,fleePath
		};
		
		for (int j=0;j<8;j++) {
			up1.addBlock(blocks[i++] = new RectBlock(160,80+j*20,BlockImage.BLACK,3));
			up1.addBlock(blocks[i++] = new RectBlock(192,80+j*20,BlockImage.BLACK,3));
			up2.addBlock(blocks[i++] = new RectBlock(64,80+j*20,BlockImage.BLACK,3));
			up2.addBlock(blocks[i++] = new RectBlock(288,80+j*20,BlockImage.BLACK,3));
			up3.addBlock(blocks[i++] = new RectBlock(32,80+j*20,BlockImage.BLACK,3));
			up3.addBlock(blocks[i++] = new RectBlock(320,80+j*20,BlockImage.BLACK,3));
		}
		
		for (int j=0;j<8;j++) {
			blocks[i++] = new RectBlock(96,80+j*20,BlockImage.BLACK,3);
			blocks[i++] = new RectBlock(128,80+j*20,BlockImage.BLACK,3);
			blocks[i++] = new RectBlock(224,80+j*20,BlockImage.BLACK,3);
			blocks[i++] = new RectBlock(256,80+j*20,BlockImage.BLACK,3);
		}

		lf2.addBlock(blocks[i++] = new RectBlock(32,240,BlockImage.BLACK,3));
		lf2.addBlock(blocks[i++] = new RectBlock(64,240,BlockImage.BLACK,3));
		lf2.addBlock(blocks[i++] = new RectBlock(96,240,BlockImage.BLACK,3));
		lf1.addBlock(blocks[i++] = new RectBlock(128,240,BlockImage.BLACK,3));
		rt1.addBlock(blocks[i++] = new RectBlock(224,240,BlockImage.BLACK,3));
		rt2.addBlock(blocks[i++] = new RectBlock(256,240,BlockImage.BLACK,3));
		rt2.addBlock(blocks[i++] = new RectBlock(288,240,BlockImage.BLACK,3));
		rt2.addBlock(blocks[i++] = new RectBlock(320,240,BlockImage.BLACK,3));
		
		RectBlock b1 = new RectBlock(160,240,BlockImage.BLACK,3);
		RectBlock b2 = new RectBlock(192,240,BlockImage.BLACK,3);

		dw1.addBlock(blocks[i++] = b1);
		dw1.addBlock(blocks[i++] = b2);

		face = new Block[8];
		face[0] = blocks[i++] = new RectBlock(96,120,BlockImage.ORANGE);
		face[1] = blocks[i++] = new RectBlock(128,120,BlockImage.ORANGE);
		face[2] = blocks[i++] = new RectBlock(224,120,BlockImage.ORANGE);
		face[3] = blocks[i++] = new RectBlock(256,120,BlockImage.ORANGE);
		face[4] = blocks[i++] = new RectBlock(128,200,BlockImage.ORANGE);
		face[5] = blocks[i++] = new RectBlock(160,220,BlockImage.ORANGE);
		face[6] = blocks[i++] = new RectBlock(192,220,BlockImage.ORANGE);
		face[7] = blocks[i++] = new RectBlock(224,200,BlockImage.ORANGE);
		
		angrydw.addBlock(face[4]);
		angryup.addBlock(face[5]);
		angryup.addBlock(face[6]);
		angrydw.addBlock(face[7]);
		
		for (Block eye:face) {
			eye.setVisible(false);
		}
		
		textBlock = new TextBlock("You'll never find my weakness, har har har");
		blocks[i++] = textBlock;
		textBlock.setBlockX((model.getWidth()-textBlock.getBlockWidth())/2);
		textBlock.setBlockY(10);
		textBlock.setVisible(false);
		
		// No Items are to be droppeed
		EmptyBehavior bh = new EmptyBehavior();
		for (int j=0;j<i;j++) {
			blocks[j].setRock(true);
			blocks[j].setBehavior(bh);
		}
		itemBlocks = new Block[] {b1,b2};
		ActionBehaviour bha = new ActionBehaviour(itemBlocks);
		b2.setBehavior(bha);
		b1.setBehavior(bha);
		model.addObserver(new Observer() {
			@Override
			public void update(Observable o, Object arg) {
				BallEvent event = (BallEvent) arg;
				if (event.getType()==TYPE.LIFE_LOST) {
					checkItemBlocks();
				}
			}
		});
	}
	
	private void showEye() {
		if (faceShow) return;
		else faceShow = true;
		for (TriggeredPath p:triggers) {
			p.trigger();
		}
		for (Block b:face) {
			b.setOpacity(0f);
			b.setVisible(true);
		}
		textBlock.setOpacity(0f);
		textBlock.setVisible(true);
		new Thread() {
			public void run() {
				try {
					for (int i=0;i<20;i++) {
						float opc = 1f/20*i;
						for (Block eye:face) {
							eye.setOpacity(opc);
						}
						textBlock.setOpacity(opc);
						Thread.sleep(100);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}.start();
	}
	
	private void becomeAngry() {
		if (angry) return;
		else angry = true;
		for (TriggeredPath t:angryTriggers) {
			t.trigger();
		}
		textBlock.setText("Grrrrrrrrr");
		
		new Thread() {
			public void run() {
				try {
					Thread.sleep(1000);
					droppingItem = ItemType.ORANGE;
					Thread.sleep(2000);
					textBlock.setText("Argl..., but Vajanoss will come again!");
				} catch (InterruptedException e) {
					e.printStackTrace();
				} finally {
					droppingItem = ItemType.ORANGE;
				}
			}
		}.start();
	}
	
	@Override
	public void start() {
		super.start();
		time = 0;
	}
	
	@Override
	public void step(long dt) {
		super.step(dt); // do the path' work
		time += dt;
		if (!faceShow && time>=3000) {
			showEye();
		}
		if (!texted &&!angry && time>15000) {
			texted = true;
			textBlock.setText("Ha ha ha, I am invincible!");
		}
		if (!texted2 &&!angry && time>30000) {
			texted2 = true;
			textBlock.setText("My mouth remains closed!");
		}
		if (faceShow && !angry) {
			int count = model.getBallCount();
			if (count>400) {
				becomeAngry();
			}
		}
	}
	
	private void checkItemBlocks() {
		if (itemBlocks[0].isDestroyed() &&
				itemBlocks[1].isDestroyed()) {
			for (Block b:itemBlocks) {
				b.setDestroyed(false);
				b.setVisible(true);
			}
		}
	}
	
	@Override
	public boolean isWon() {
		for (Block b:blocks) {
			if (b!=null && !b.isDestroyed()) {
				return false;
			}
		}
		return true;
	}
	
	@Override
	public Path[] getPaths() {
		return paths;
	}

	@Override
	public Block[] getBlocks() {
		return blocks;
	}

	@Override
	public String getName() {
		return "Vajanoss";
	}
	
	private void launchItem() {
		int x = new Random().nextInt(model.getWidth());
		model.notifyObservers(new ItemReleasedEvent(x, 0, droppingItem));
	}
	
	private class ActionBehaviour extends DefaultBehavior {
		
		private boolean available = true; 
		private Block[] blocks;
		
		private ActionBehaviour(Block[] blocks) {
			this.blocks = blocks;
		}
		
		public void blockStroked(int x, int y, Eventable eventable) {
			if (!available) return;
			available = false;
			launchItem();
			new Thread() {
				public void run() {
					try {
						for (Block b:blocks)
							b.setOpacity(.5f);
						Thread.sleep(3000);
					} catch (Exception e) {
						e.printStackTrace();
					} finally {
						available = true;
						for (Block b:blocks)
							b.setOpacity(1f);
					}
				}
			}.start();
		}

		public void blockDestroyed(int x, int y, int hp, Eventable eventable) {
			blockStroked(x,y,eventable);
		}
	}
}
