package ch.lanter.vajanoid.level;

import ch.lanter.vajanoid.Model;

/**
 * The levelchain defines the order of the levels
 * 
 * @author Martin Lanter, Lantersoft 2010
 */
public class LevelChain {

	// all the levels which are to be played
	private Level[] levels;
	
	// pointer to the current level
	private int ptr;
	
	public LevelChain(Model model) {
		int i = 0;
		levels = new Level[30];
		levels[i++] = new Level01(); // Welchome
		levels[i++] = new Level02(); // Square_
		levels[i++] = new Level03(); // Halloween
		levels[i++] = new Level04(); // Clueless_
		levels[i++] = new Level05(); // Shuttle
		levels[i++] = new Level06(); // Death Sprial_
		levels[i++] = new Level07(); // Starfish
		levels[i++] = new Level08(); // Conveyer_
		levels[i++] = new Level09(); // Rainbow
		levels[i++] = new Level10(); // Infinity_
		levels[i++] = new Level11(model); // ETH
		levels[i++] = new Level12(); // Yellow_
		levels[i++] = new Level13(); // Sunray
		levels[i++] = new Level14(); // Carnival_
		levels[i++] = new Level15(); // Blocky
		levels[i++] = new Level16(); // Highway_
		levels[i++] = new Level17(); // Black Hole
		levels[i++] = new Level18(model); // Repulsion_
		levels[i++] = new Level19(); // Bisector
		levels[i++] = new Level20(); // Switzerland_
		levels[i++] = new Level21(); // Tile
		levels[i++] = new Level22(); // Circle_
		levels[i++] = new Level23(); // Packman
		levels[i++] = new Level24(model); // Button Anger_
		levels[i++] = new Level25(); // Despair
		levels[i++] = new Level26(); // Goldorpitch_
		levels[i++] = new Level27(model); // Quicky
		levels[i++] = new Level28(); // Elevator_
		levels[i++] = new Level29(); // La Fin
		levels[i++] = new Level30(model); // Vajanoss_
		ptr = -1;
	}
	
	/**
	 * Skip jmp levels and return the new current level
	 */
	public Level getNext(int jump) {
		ptr += jump;
		if (ptr<levels.length) {
			return levels[ptr];
		} else {
			return null;
		}
	}
	
	/**
	 * @return - true if there is a successive level
	 */
	public boolean hasNext() {
		return ptr+1<levels.length;
	}
	
	/**
	 * @return - true if the last level has been finished. Some levels might have been skipped
	 */
	public boolean hasFinished() {
		return ptr>=levels.length;
	}
	
	public int getNumber() {
		return ptr;
	}
	
	/**
	 * Evaluate a levelcode. Return 0 if code is an invalid level code
	 * @param code - level code
	 * @return - how many levels must be skipped to get to the desired level.
	 */
	public int getJumpsToLevel(String code) {
		for (int i=ptr;i<levels.length;i++) {
			if (levels[i].getName().toLowerCase().trim().equals(code.trim())) {
				return i-ptr;
			}
		}
		return 0;
	}
}
