package ch.lanter.vajanoid.net;

import java.awt.AlphaComposite;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.TimingTargetAdapter;

/**
 * Displays the ranking 
 * 
 * @author Martin Lanter, Lantersoft 2010
 */
public class PnlRank extends JLabel {

	private static final long serialVersionUID = 200911080133L;
	
	private float opacity = 0f; // will be .95
	
	private JTable tbl;
	private JTextField txf; // to get the name
	
	private Ranking ranking;
	private Rank rank; // the highscore of the player
	
	public PnlRank(final Ranking ranking, Rank rank) {
		this.ranking = ranking;
		this.rank = rank;
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(40,20,40,20));
		setOpaque(false);
		
		Rank[] ranks = ranking.getRanks();
		final RankModel mdl = new RankModel(ranks, rank);
		this.tbl = new JTable(mdl) {
			private static final long serialVersionUID = 200912050252L;
			public Component prepareRenderer( TableCellRenderer renderer, int row, int column ) {
				Component c = super.prepareRenderer( renderer, row, column );
				if (row==mdl.getRankIndex())
					c.setBackground(Color.GREEN);
				else if (row==0) c.setBackground(Color.YELLOW);
				else if (row==1) c.setBackground(Color.ORANGE);
				else if (row==2) c.setBackground(Color.ORANGE);
				else c.setBackground(Color.WHITE);
				return c;
			}
		};
		tbl.setOpaque(false);
		tbl.setFont(tbl.getFont().deriveFont(Font.BOLD));
		JScrollPane scr = new JScrollPane(tbl);
		scr.setOpaque(false);
		scr.getViewport().setOpaque(false);
		scr.getVerticalScrollBar().setOpaque(false);
		scr.setBorder(new EmptyBorder(0,0,0,0));
		add(scr,BorderLayout.CENTER);
		
		TableColumnModel tcm = tbl.getColumnModel();
		tcm.getColumn(0).setPreferredWidth(118); // sum = 340-2*20
		tcm.getColumn(1).setPreferredWidth(52);
		tcm.getColumn(2).setPreferredWidth(32);
		tcm.getColumn(3).setPreferredWidth(100);
		
		final int ri = mdl.getRankIndex();
		tbl.setSelectionMode( ListSelectionModel.SINGLE_SELECTION  );
		tbl.setColumnSelectionAllowed( true );
		tbl.setRowSelectionAllowed( true );
		tbl.changeSelection( ri ,0, false, false );
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				tbl.editCellAt(ri,0);
				String value = (String) tbl.getValueAt(ri, 0);
				TableCellEditor cedit = tbl.getCellEditor(ri,0);
				txf = (JTextField) cedit.getTableCellEditorComponent(
						tbl, value, true, ri, 0);
				txf.getCaret().setSelectionVisible(true);
				txf.selectAll();
			}
		});
		
		JPanel pnlButtons = new JPanel(new GridLayout(1,3));
		pnlButtons.setOpaque(false);
		pnlButtons.setBorder(new EmptyBorder(10,10,10,10));
		pnlButtons.add(new JLabel());
		JButton btnO = new JButton("Submit");
		JButton btnC = new JButton("Cancel");
		btnO.addActionListener(new ActionListener() {
			private boolean done = false;
			public void actionPerformed(ActionEvent e) {
				if (done) return; // only one singe submit allowed
				submit(txf.getText());
				if (ranking.isUpSuccessful()) {
					done = true;
					setVisible(false);
				} else {
					JOptionPane.showMessageDialog(
							null, 
							"Rank upload to www.lantersoft.ch has failed. Please check whether your internet connection is intact and try again.", 
							"Failed rank transmission", JOptionPane.WARNING_MESSAGE);
				}
			}
		});
		btnC.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}
		});
		pnlButtons.add(btnO);
		pnlButtons.add(btnC);
		add(pnlButtons,BorderLayout.SOUTH);
		this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
	
	private void submit(Object name) {
		Rank r = new Rank(
				name.toString(),
				rank.getScore(),
				rank.getLevel(),
				rank.getDate());
		ranking.uploadRank(r);
	}
	
	public void appear() {
		Animator animator = new Animator(2000);
		animator.addTarget(new TimingTargetAdapter() {
			public void timingEvent(float fraction) {
				opacity = .95f * fraction;
				repaint();
			}
			
			public void begin() {
				tbl.requestFocus();
			}
			
			public void end() {
				opacity = .95f;
				repaint();
			}
		});
		animator.setAcceleration(.3f);
		animator.setDeceleration(.3f);
		animator.setResolution(50);
		animator.start();
	}
	
	@Override
    public void paint(Graphics g) {
		Rectangle clip = g.getClipBounds();
        BufferedImage image = new BufferedImage(clip.width,clip.height,BufferedImage.TYPE_INT_ARGB);
        Graphics2D ig = (Graphics2D) image.getGraphics();
        ig.setClip(0,0,clip.width,clip.height);
        ig.setTransform(AffineTransform.getTranslateInstance(-clip.x,-clip.y));
        super.paint(ig);
        Graphics2D g2 = (Graphics2D) g.create();
        g2.setComposite(AlphaComposite.SrcOver.derive(opacity));
        g2.drawImage(image,clip.x,clip.y,this);
        g2.dispose();
    }
	
	private class RankModel extends DefaultTableModel {
		private static final long serialVersionUID = 200912050208L;
		
		private Rank[] ranks;
		private Rank rank;
		private int rankIndex;
		private int n;
		private RankModel(Rank[] ranks, Rank rank) {
			super();
			this.ranks = ranks;
			this.rank = rank;
			this.n = ranks.length;
			this.rankIndex = n;
			for (int i=0;i<n;i++) {
				if (rank.getScore()>ranks[i].getScore()) {
					rankIndex = i;
					break;
				}
			}
		}
		
		private int getRankIndex() {
			return rankIndex;
		}
		
		@Override
		public boolean isCellEditable(int row, int column) {
			return row==rankIndex && column==0;
		}

		@Override
		public String getColumnName(final int column) {
			switch(column) {
			case 0: return "Name";
			case 1: return "Score";
			case 2: return "Level";
			case 3: return "Date";
			default: return null;
			}
		}

		@Override
		public int getColumnCount() {
			return 4;
		}

		@Override
		public int getRowCount() {
			return n+1;
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			Rank r;
			if (rowIndex==rankIndex) {
				r = rank;
			} else {
				if (rowIndex>rankIndex) rowIndex--;
				r = ranks[rowIndex];
			}
			switch(columnIndex) {
			case 0: return r.getName();
			case 1: return r.getScore();
			case 2: return r.getLevel();
			case 3: return r.getDate();
			default: return null;
			}
		}
		
		@Override
		public void setValueAt(Object aValue, int row, int column) {
			if (row==rankIndex && column==0) {
				rank = new Rank(aValue.toString(),
						rank.getScore(),
						rank.getLevel(),
						rank.getDate());
				tbl.changeSelection( row ,0, false, false );
			}
			
		}
	}
}
