package ch.lanter.vajanoid.net;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * Provides functionality to download the ranking and upload the highscore
 *
 * @author Martin Lanter, Lantersoft 2010
 */
public class Ranking {

	private static final long serialVersionUID = 200911060300L;

	private boolean successDown = false;
	private boolean successUp = true;
	private ArrayList<Rank> list = new ArrayList<Rank>();

	/**
	 * Should be startet in a new Thread to not delay any other pending
	 * processes.
	 */
	public Ranking() {
		downloadRanking();
	}

	public boolean isDownSuccessful() {
		return successDown;
	}

	public boolean isUpSuccessful() {
		return successUp;
	}

	public Rank[] getRanks() {
		return list.toArray(new Rank[list.size()]);
	}

	/**
	 * Download the ranking from lantersoft.ch
	 */
	public void downloadRanking() {
		successDown = false;
		try {
			URL url = new URL( "http://www.lantersoft.ch/de/vajanoid/access.php");
		    Scanner scanner = new Scanner( url.openStream() ).useDelimiter( "\n" );
		    while(scanner.hasNext()) {
		    	String line = scanner.next();

		    	String[] elements = line.split(";");
		    	String name = elements[0];
		    	int score = Integer.parseInt(elements[1]);
		    	int level = Integer.parseInt(elements[2]);
		    	String temp = elements[3];
		    	String date = temp.substring(0,temp.length()-3);

		    	Rank rank = new Rank(name,score,level,date);
		    	list.add(rank);
		    }
		    successDown = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Upload the given rank to lantersoft.ch
	 * Note: Please don't mess around with this method. I know it's tempting to
	 * upload a crazy highscore and put yourself on top of the ranking but it's
	 * not that impressive, since I'm showing you the source code ond not fair
	 * to me or other players either. Thank you :)
	 */
	public void uploadRank(Rank rank) {
		/*
		successUp = false;

		String name = validate(rank.getName());
		String entry  = "http://www.lantersoft.ch/de/vajanoid/upload.php?" +
			"score="+rank.getScore()+
			"&level="+rank.getLevel()+
			"&name="+name;
		try {
			URL u = new URL(entry);
			u.openStream();
			successUp = true;
		} catch (IOException e) {
			e.printStackTrace();
		}
		*/
	}

	public String validate(String entry) {
		entry = entry.trim();
		entry = entry.replace(';',',');
		entry = entry.replace('<','[');
		entry = entry.replace('>',']');
		entry = entry.replace('(','[');
		entry = entry.replace(')',']');
		entry = entry.replace('{','[');
		entry = entry.replace('}',']');
		entry = entry.replace('&','+');
		entry = entry.replace('=','-');
		entry = entry.replace('?','#');
		entry = entry.replace(' ','_');
		return entry;
	}
}
