package ch.lanter.vajanoid.path;

import java.util.ArrayList;
import java.util.Iterator;

import ch.lanter.vajanoid.block.Block;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class CirclePath extends AbstractPath {

	private float cx, cy; // center
	private float r; // radius
	private float v; // angel velocity

	/*
	 * Must have the same length as the ArrayList blocks from the superclass
	 */
	private ArrayList<Angle> angles; // achieved angles
	
	public CirclePath(float cx, float cy, float r, float v) {
		this.cx = cx;
		this.cy = cy;
		this.r = r;
		this.v = v;
		angles = new ArrayList<Angle>();
	}
	
	public void arrange() {
		int n = getCount();
		int i = 0;
		Iterator<Block> itrB = getBlockIterator();
		angles.clear();
		while (itrB.hasNext()) {
			Block block = itrB.next();
			double angle = 2*Math.PI*i++/n;
			block.setLocation(
					(int) Math.round(cx+r*Math.cos(angle))-block.getBlockWidth()/2,
					(int) Math.round(cy+r*Math.sin(angle))-block.getBlockHeight()/2 );
			angles.add(new Angle(angle));
		}
	}

	@Override
	public void step(long dt) {
		float dphi = dt*v/1000;

		Iterator<Block> itrB = getBlockIterator();
		Iterator<Angle> itrA = angles.iterator();
		while (itrB.hasNext()) {
			Block block = itrB.next();
			Angle angle = itrA.next();
			double phi = angle.angle;
			block.setLocation(
					(int) (Math.round(cx+r*Math.cos(phi+dphi))-block.getBlockWidth()/2), 
					(int) (Math.round(cy+r*Math.sin(phi+dphi))-block.getBlockHeight()/2));
			angle.angle = phi+dphi;
		}
	}
	
	@Override
	public void addBlock(Block block) {
		angles.add(new Angle(0));
		super.addBlock(block);
	}
	
	private static class Angle {
		private double angle;
		private Angle(double angle) {
			this.angle = angle;
		}
	}

}
