package ch.lanter.vajanoid.path;

import java.awt.Rectangle;
import java.util.Iterator;

import ch.lanter.vajanoid.Ball;
import ch.lanter.vajanoid.Model;
import ch.lanter.vajanoid.Shot;
import ch.lanter.vajanoid.block.Block;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class FleePath extends AbstractPath {

	private Model model;
	
	private float v;
	
	/**
	 * Constructor
	 * @param model - the Modell
	 * @param v - the velocity for the blocks. 6f ist not too bad.
	 */
	public FleePath(Model model, float v) {
		this.model = model;
		this.v = v;
	}

	@Override
	public void step(long dt) {
		Ball[] balls = model.getBalls();
		Shot[] shots = model.getShots();
		Block[] blocks = model.getLevel().getBlocks();
		float c = 100f;

		Iterator<Block> itrB = getBlockIterator();
		while (itrB.hasNext()) {
			Block block = itrB.next();
			float x = block.getBlockX()+block.getBlockWidth()/2;
			float y = block.getBlockY()+block.getBlockHeight()/2;
			
			for (int i=0;i<balls.length;i++) {
				int bx = balls[i].getX()+balls[i].getWidth()/2;
				float by = balls[i].getY()+balls[i].getHeight()/2;
				float r2 = ((x-bx)*(x-bx)+(y-by)*(y-by));
				x = x+(x-bx)/r2*v*c;
				y = y+(y-by)/r2*v*c;
			}
			for (int i=0;i<shots.length;i++) {
				int sx = shots[i].getX()+shots[i].getWidth()/2;
				float sy = shots[i].getY()+shots[i].getHeight()/2;
				float r2 = ((x-sx)*(x-sx)+(y-sy)*(y-sy));
				x = x+(x-sx)/r2*v*c;
				y = y+(y-sy)/r2*v*c;
			}
			int xx = Math.round(x);
			int yy = Math.round(y);
			
			if (xx<16) xx = 16;
			if (yy<10) yy = 10;
			if (xx>368) xx = 368;
			if (yy>270) yy = 270;
			
			boolean clear = true;
			Rectangle rect = new Rectangle(xx-16,yy-10,32,20);
			for (Block b:blocks) {
				if (b!=null && b!=block)
					clear &= !b.getBounds().intersects(rect);
			}
			
			if (!clear) { // horizontal
				clear = true;
				rect = new Rectangle(xx-16,block.getBlockY(),32,20);
				for (Block b:blocks) {
					if (b!=null && b!=block)
						clear &= !b.getBounds().intersects(rect);
				}
			}
//			
			if (!clear) { // vertical
				clear = true;
				rect = new Rectangle(block.getBlockX(),yy-10,32,20);
				for (Block b:blocks) {
					if (b!=null && b!=block)
						clear &= !b.getBounds().intersects(rect);
				}
			}
			
			if (clear) {
				block.setBlockX(rect.x);
				block.setBlockY(rect.y);
			}
			
		}
	}

}
