package ch.lanter.vajanoid.path;

import java.util.ArrayList;
import java.util.Iterator;

import ch.lanter.vajanoid.block.Block;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class PolarPath extends AbstractPath {

	private float cx, cy;
	private float v;
	
	private ArrayList<Polar> polars;
	
	public PolarPath(float cx, float cy, float v) {
		this.cx = cx;
		this.cy = cy;
		this.v = v;
		this.polars = new ArrayList<Polar>();
	}
	
	public void arrange() {
		step(0);
	}

	@Override
	public void step(long dt) {
		float dphi = dt*v/1000;

		Iterator<Block> itrB = getBlockIterator();
		Iterator<Polar> itrA = polars.iterator();
		while (itrB.hasNext()) {
			Block block = itrB.next();
			Polar polar = itrA.next();
			double phi = polar.angle;
			float r = polar.radius;
			block.setLocation(
					(int) (Math.round(cx+r*Math.cos(phi+dphi))-block.getBlockWidth()/2), 
					(int) (Math.round(cy+r*Math.sin(phi+dphi))-block.getBlockHeight()/2));
			polar.angle = polar.angle+dphi;
		}
	}
	
	@Override
	public void addBlock(Block block) {
		int x = block.getBlockX();
		int y = block.getBlockY();
		float radius = (float)Math.sqrt(x*x+y*y);
		float angle =(float) Math.atan2(y,x);
		this.addBlock(block, radius, angle);
	}
	
	public void addBlock(Block block, float radius, float angle) {
		super.addBlock(block);
		polars.add(new Polar(radius,angle));
	}
	
	private static class Polar {
		private float radius, angle;
		private Polar(float radius, float angle) {
			this.radius = radius;
			this.angle = angle;
		}
	}

}
