package ch.lanter.vajanoid.path;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;

import ch.lanter.vajanoid.block.Block;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public class ShiftPath extends TriggeredPath {

	private float s,v;
	private double ex,ey; // error
	private int ptr;
	private ArrayList<Point> vectors; // Vectors pointing the way
	private float[] dists; // dist[n] is the total way
	
	public ShiftPath(float v) {
		this.v = v;
		this.vectors = new ArrayList<Point>();
	}

	@Override
	public void step(long dt) {
		if (!isTriggered()) return;
		if (isFinished()) return;
		float ds = dt*v/1000;
		
		double ddx = 0;
		double ddy = 0;
		
		// Increse the ptr until it points to the next higher dist than s+ds
		while (s+ds>dists[ptr]) {
			Point p = vectors.get(ptr);
			double c = (dists[ptr]-s)/Math.sqrt(p.x*p.x+p.y*p.y);
			ddx += c*p.x;
			ddy += c*p.y;
			ds = ds-(dists[ptr]-s);
			s = dists[ptr];
			ptr++;
			if (ptr==dists.length) {
				translate((int) Math.round(ddx), (int) Math.round(ddy));
				finish();
				return;
			}
		}
		
		Point p = vectors.get(ptr);
		double c = ds/Math.sqrt(p.x*p.x+p.y*p.y);
		ddx += c*p.x - ex;
		ddy += c*p.y - ey;
		
		int dx = (int) Math.round(ddx);
		int dy = (int) Math.round(ddy);
		ex = dx-ddx;
		ey = dy-ddy;
		
		translate(dx,dy);
		
		s = s + ds;
	}
	
	private void translate(int dx, int dy) {
		Iterator<Block> iterator = getBlockIterator();
		while(iterator.hasNext()) {
			Block block = iterator.next();
			block.setLocation(
					block.getBlockX()+dx,
					block.getBlockY()+dy);
		}
	}
	
	@Override
	public void trigger() {
		int n = vectors.size();
		if (n==0) throw new IllegalStateException("ShiftPath has no vectors");
		dists = new float[n];
		dists[0] = (float)Math.sqrt(vectors.get(0).x*vectors.get(0).x+vectors.get(0).y*vectors.get(0).y);
		for (int i=1;i<n;i++) {
			int dx = vectors.get(i).x;
			int dy = vectors.get(i).y;
			dists[i] = dists[i-1] + (float)Math.sqrt(dx*dx+dy*dy/*1.6*1.6*/);
		}
		super.trigger();
	}
	
	public void addVector(Point p) {
		vectors.add(p);
	}

}
