package ch.lanter.vajanoid.path;

/**
 * @author Martin Lanter, Lantersoft 2010
 */
public abstract class TriggeredPath extends AbstractPath {

	private TriggeredPath path;
	
	private boolean triggered = false;
	private boolean finished = false;

	/**
	 * Should be called from overwritten trigger() methods from subclasses to
	 * ensure the proper handling with the private flag triggered.
	 */
	public void trigger() {
		triggered = true;
	}

	/**
	 * Should be called from overwritten finish() methods from subclasses to
	 * ensure the proper handling with the private flag finished and for that
	 * the next TriggerPath gets triggered
	 */
	public void finish() {
		finished = true;
		if (path!=null)
			path.trigger();
	}
	
	public boolean isFinished() {
		return finished;
	}
	
	public boolean isTriggered() {
		return triggered;
	}

	/**
	 * Set a Path, which will be triggered as soon as finish() is called
	 * @param path - the next Triggerpath to be triggered
	 */
	public void setFollowingPath(TriggeredPath path) {
		this.path = path;
	}

}
